#include <xc.h>
#include <pic16f886.h>
#include "lcd_sub.h"
#include "port_init.h"
#include "wait_time.h"
#include "eeprom.h"
#include "user_define.h"
#include "preset.h"
#include "ad.h"
#include "usart.h"

#pragma config BOREN = OFF
#pragma config CPD = OFF 
#pragma config FOSC =INTRC_NOCLKOUT  
#pragma config MCLRE = ON 
#pragma config WDTE = OFF 
#pragma config CP = OFF 
#pragma config LVP = OFF 
#pragma config PWRTE = ON
#pragma config IESO =OFF
#pragma config FCMEN = OFF

#pragma config BOR4V = BOR21V
#pragma config WRT = OFF

unsigned char	bank_no;
unsigned int	khz_no;
unsigned int	khz_no_m;
unsigned char	mode;
unsigned int	khz_no_setmode;

unsigned char	preset_no;
unsigned int	preset[PRESET_MAX];
unsigned char	preset_no_setmode;

unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw3_off;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;

unsigned char	sw1p;
unsigned char	sw2p;
unsigned char	sw3p;
unsigned char	sw4p;
unsigned char	tun_flg;
unsigned int	kyoku_count;
unsigned int	vol_count;
unsigned int	rqst_rssi;
unsigned char	change;
unsigned char	st_flg;


unsigned char	receive_data;
unsigned int	receive_flg;	/* 1:M 0:M */
unsigned char	data_rssi;

unsigned char	width;
unsigned char	wid_flg;

unsigned int    freq_no;

/*--------------------USART INTERRUPT-------------*/
void	interrupt InterReceiver(void)
{
	if(RCIF==1){
		receive_data=RCREG;
		receive_flg=1;
		RCIF =0;
	}
}
/*---------------------USART TX--------------------*/
void	tune_tx(unsigned int data)
{
unsigned char	no1,no2;
unsigned int    no;
	
    no=data >> 8;
	no1=(unsigned char)no;
    no=data & 0xff;
    no2=(unsigned char)no;
    
	putch(no1);
    putch(no2);
}

void	data_tx(unsigned char data)
{
	putch(0x00);
    putch(data);
}
/*---------------------MAIN-----------------------*/
void	main(void)
{

	port_init();

	__delay_ms(2000);

	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;sw3_off=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;
	mode=1;preset_no=0;
	tun_flg=1;
	st_flg=0;
	width=2;
	wid_flg=0;
    freq_no=0;

	receive_flg=0;
	init_usart();

	lcd_init();
	lcd_clear();

	while(1){
	//****************************
	change=0;

	sw1p=RC0;NOP();
	sw2p=RC1;NOP();
	sw3p=RC2;NOP();
	sw4p=RC5;NOP();

	if((sw2p==0)&(sw3p==0)){
		eeprom_init();
	}
	else{
		if((sw1p==0)&(sw2p==1)&(sw3p==1)){
			bank_no=1;
			E2promWrite(0x0c,bank_no);
		}
		if((sw1p==1)&(sw2p==0)&(sw3p==1)){
			bank_no=0;
			E2promWrite(0x0c,bank_no);
		}
		eeprom_check();
	}

	lcd_xy(0,0);
	lcd_print("SW RADIO");
	lcd_xy(0,1);
	lcd_print("        ");

	if(st_flg==0){
		lcd_xy(5,1);lcd_print("BK");
		lcd_su(bank_no);
		st_flg=1;
	}

	__delay_ms(6000);
	lcd_ini_disp();
	lcd_xy(0,1);lcd_print("        ");

	switch(mode){
		case 0:
				khz_no=preset[preset_no];
				lcd_xy(0,0);lcd_khz(khz_no);
				tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
				lcd_mode(mode);
				break;
		case 1:	if(tun_flg==1){
					khz_no=vol_yomi();
				}
				khz_no_m=khz_no;
				lcd_xy(0,0);lcd_khz(khz_no);
				tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
				lcd_ch_disp();
				lcd_mode(mode);
				break;
	}

	while(change==0){

		sw1p=RC0;NOP();
		sw2p=RC1;NOP();
		sw3p=RC2;NOP();
		sw4p=RC5;NOP();
	/* key scan 1 */
		if (sw1p==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (sw2p==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (sw3p==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
		}
		/*------------------*/
		if (sw4p==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/

		wait_time(1000);

		sw1p=RC0;NOP();
		sw2p=RC1;NOP();
		sw3p=RC2;NOP();
		sw4p=RC5;NOP();
	/* key scan 2 */
		if ((sw1p==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((sw2p==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((sw3p==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
			if(sw3_cnt>0){
				sw3_off=1;
			}
		}
		/*------------------*/
		if ((sw4p==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/
		if(receive_flg==1){
			data_rssi=receive_data;
			receive_flg=0;
		}
	/* key action */
	
		if((sw1_key==0)&&(sw1_rep==0)){
			switch(mode){
				case 0:
						sw1_rep=1;
						preset_no=preset_next(preset_no);
						khz_no=preset[preset_no];
						lcd_xy(0,0);
						lcd_khz(khz_no);
						tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
						lcd_mode(mode);
						sw1_cnt=0;
						write_direct();
						break;
				case 1:	
						sw1_rep=1;
						if(freq_no>=23){
                            freq_no=0;
                        }
                        else{
                            freq_no++;
                        }
						khz_no=vol_yomi();
                        lcd_xy(0,0);
                        lcd_khz(khz_no);
                        tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
                        lcd_mode(mode);
                        sw1_cnt=0;
                        write_direct();
						break;
				case 2:
						sw1_rep=1;
						preset_no_setmode++;
						if(preset_no_setmode>=PRESET_MAX){preset_no_setmode=1;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(3,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
						}
						else{
							lcd_print(" NONE");
						}
						sw1_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*------------------*/

		if((sw2_key==0)&&(sw2_rep==0)){
			switch(mode){
				case 0:
						sw2_rep=1;
						preset_no=preset_back(preset_no);
						khz_no=preset[preset_no];
						lcd_xy(0,0);
						lcd_khz(khz_no);
						tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
						lcd_mode(mode);
						sw2_cnt=0;
						write_direct();
						break;
				case 1:
						sw2_rep=1;
						if(freq_no==0){
                            freq_no=23;
                        }
                        else{
                            freq_no--;
                        }
						khz_no=vol_yomi();
                        lcd_xy(0,0);
                        lcd_khz(khz_no);
                        tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
                        lcd_mode(mode);
                        sw2_cnt=0;
                        write_direct();                        
						break;
				case 2:
						sw2_rep=1;
						if(preset_no_setmode<=1){preset_no_setmode=PRESET_MAX-1;}
						else{preset_no_setmode--;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(3,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
						}
						else{
							lcd_print(" NONE");
						}
						sw2_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			sw3_rep=1;
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==1)){
			sw3_cnt++;
		}
		/*--------------------*/
		if((sw3_key==1)&&(sw3_off==1)){
			if(sw3_cnt>=40){
				/**/
				switch(mode){
					case 0:	break;
					case 1:
							mode=2;
							preset_no_setmode=preset_no;
							khz_no_setmode=preset[preset_no];
							lcd_xy(3,1);
							lcd_khz(khz_no_setmode);
							sw1_cnt=0;
							lcd_mode(mode);
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=1;
								preset_no=preset_no_setmode;
								preset[preset_no]=0;
								if(tun_flg==1){
									khz_no=vol_yomi();
								}
								lcd_xy(0,0);
								lcd_khz(khz_no);
								tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
								lcd_ch_disp();
								lcd_mode(mode);
								write_direct();
								write_direct_set(preset_no);
								preset_no=preset_next(preset_no);
							}
							break;
				}
			}
			else{
				/*Z*/
				switch(mode){
					case 0:
							mode=1;
							if(tun_flg==1){
								khz_no=vol_yomi();
							}
							lcd_xy(0,0);
							lcd_khz(khz_no);
							tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
							lcd_ch_disp();
							lcd_mode(mode);
							write_direct();
							break;
					case 1:
							mode=0;
							khz_no=preset[preset_no];
							lcd_xy(0,0);
							lcd_khz(khz_no);
							tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
							lcd_mode(mode);
							write_direct();
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=0;
								preset_no=preset_no_setmode;
								preset[preset_no]=khz_no;
								lcd_xy(0,0);
								lcd_khz(khz_no);
								tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
								lcd_mode(mode);
								write_direct();
								write_direct_set(preset_no);
							}
							break;
				}
			}
			
			sw3_off=0;
			sw3_cnt=0;
		}
		/*--------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			switch(mode){
				case 0:	
                        sw4_rep=1;
						switch(wid_flg){
							case 0:	wid_flg=1;
									lcd_width();
									rqst_rssi=50;
									break;
							case 1:	rqst_rssi=50;
									width++;
									if(width>4){width=0;}
									lcd_width();
									data_tx(width);
									write_direct_width();
									break;
						}                    
                        break;
				case 1:
						sw4_rep=1;
						break;
				case 2:	sw4_rep=1;
						mode=1;
						lcd_xy(0,0);
						lcd_khz(khz_no);
						tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
						lcd_ch_disp();
						lcd_mode(mode);
						write_direct();
						break;
			}
		}
		/*------------------*/

		/*------------------ */
		if(change==0){
		//**************************
		if((mode==1)&&(tun_flg==1)){
		/*----------*/
			if(vol_count>0){vol_count--;}
			if(vol_count==0){
				khz_no=vol_yomi();
				if(khz_no>FREQ_MAX){khz_no=FREQ_MAX;}

				if(khz_no_m!=khz_no){
					lcd_xy(0,0);
					lcd_khz(khz_no);
					tune_tx(khz_no);rqst_rssi=RQST_RSSI_SET;
					lcd_ch_disp();
					lcd_mode(mode);
					khz_no_m=khz_no;
				}
				vol_count=VOL_COUNT_SET;
			}
		}
		/*------------------ */
		if(rqst_rssi>0){
			if(mode<2){
				rqst_rssi--;
				if(rqst_rssi==0){
					wid_flg=0;
					rqst_rssi=RQST_RSSI_INIT;
					lcd_xy(3,1);lcd_print("  S");lcd_2keta10(data_rssi);
				}
			}
		}
		/**********************/


		//**************************************
		}

	}
	//*************************************************

	}

}